@@ -1,7 +1,6 @@
 package Syntax::Collector;
 
 use 5.008;
-use Sub::Uplevel qw/uplevel :aggressive/;
 use strict;
 use syntax qw//; # deliberate dependency.
 use Carp;
@@ -10,7 +9,7 @@ use Sub::Name qw/subname/;
 
 BEGIN {
 	$Syntax::Collector::AUTHORITY = 'cpan:TOBYINK';
-	$Syntax::Collector::VERSION   = '0.002';
+	$Syntax::Collector::VERSION   = '0.003';
 }
 
 sub import
@@ -79,8 +78,8 @@ sub import
 	my %sub;
 	$sub{import} = sub
 	{
-		my $self   = shift;
-		my $caller = caller;
+		my ($self, %args) = @_;
+		my $caller = $args{-into} || caller;
 		
 		foreach my $f (@features)
 		{
@@ -96,15 +95,15 @@ sub import
 			{
 				require_module($module);
 				$module->VERSION($version) if $version;
-				my $coderef = $module->can($use eq 'no' ? 'unimport' : 'import');
-				uplevel 1, $coderef, $module, @$everything
-					if $coderef;
+				my $coderef = $use eq 'no'
+					? eval "package $caller; my \$sub = sub { shift->unimport(\@_) };"
+					: eval "package $caller; my \$sub = sub { shift->import(\@_) };";
+				$coderef->($module, @$everything);
 			}
 		}
 		
 		if (my $after = $self->can('IMPORT'))
 		{
-			@_ = ($self);
 			goto $after;
 		}
 	};
@@ -259,6 +258,10 @@ can do this:
   __FILE__
   __END__
 
+C<IMPORT> is passed a copy of the same arguments that were passed to
+C<import>. C<import> uses named parameters, including support for an
+I<into> parameter; C<IMPORT> should probably do the same.
+
 As well as providing an C<import> method for your collection,
 Syntax::Collector also provides a C<modules> method, which can be called
 to find out which modules a collection includes. Called in list context,
@@ -267,10 +270,6 @@ C<< { module => version } >> hash.
 
 =head1 CAVEATS
 
-This module does some pretty tricky stuff with L<Sub::Uplevel> and
-C<eval>. It is possible that (especially in the case of convoluted
-OVERSTUFF), certain "use" lines may break.
-
 You should not rely on the "use" lines being processed in any
 particular order.
 
